import os
import math
import runes
import threading

# How often the mouse position will be sampled

class MouseSampler(threading.Thread):

	def __init__(self, getPosFunc, getUsecFunc, waitFunc, addPointFunc, mdown, particleUpdateFunc):
		threading.Thread.__init__(self)
		self.getPosFunc = getPosFunc
		self.getUsecFunc = getUsecFunc
		self.addPointFunc = addPointFunc
		self.waitFunc = waitFunc
		self.particleUpdateFunc = particleUpdateFunc
		
		self.mDown = mdown
		
		self.mouseSampleInterval = 0
		self.lasttime = getUsecFunc()
		self.go = True
	
	def stop(self):
		self.go = False
	
	def run(self):
	
		while self.go:
						
			time = self.getUsecFunc()
			if self.mDown.get() and ((time - self.lasttime) > self.mouseSampleInterval):
				self.particleUpdateFunc(time - self.lasttime)
				self.lasttime = time
				self.addPointFunc( self.getPosFunc() )
			
			self.waitFunc(5)
		return
		
	

class Runes:
	
	def __init__(self, runeCutoff = 0.7):
		self.loadRunes()
		self.runeCutoff = runeCutoff
	
	def loadRunes(self):
		
		self.runes = {}

		runeNames = filter(lambda s: "__rune__" in s, dir(runes))
		
		for runeName in runeNames:
			self.runes[runeName] = eval("runes." + runeName + "()")

        

	def whichRune(self, vector):
		scores = map(lambda rune: (rune.getScore(vector), rune), self.runes.itervalues())
		score, rune = max(scores)
		
		if score < self.runeCutoff:
			return None
		else:
			return rune
		


class MotionDetect:
	
	def __init__(self, pixelRes = 10):
		
		self.cutoff = pixelRes # 5 pixel
		
		self.lastPoint = None	# elozo pont
		
		self.vectors = []	# a vektorok, ([xdir, ydir], len, (fx, fy))
		
		self.dbg = False
		self.zcounter = 0
		self.zcutoff = 1
		self.inZero = False
	
	def myLen(self, vec):
		return math.sqrt(vec[0]*vec[0] + vec[1]*vec[1])
	
	def getVector(self, vec):
		deg = math.atan2(vec[1], vec[0]) / math.pi * 180.0
		if deg < 0.0: deg += 360.0
		
		ofs = 22.5
		bounds = []
		results = [
			[1,-1],
			[0,-1],
			[-1,-1],
			[-1,0],
			[-1,1],
			[0,1],
			[1,1],
			[1,0],
			[1,0],
		]
		for i in xrange(7):
			bounds.append( (i*45.0 + ofs, (i+1)*45.0 + ofs) )
		bounds.append( (360.0-ofs, 360.0) )
		bounds.append( (0.0, ofs) )
		
		#print "*******"
		#for i in xrange(len(bounds)):
		#	print "bound", bounds[i], results[i]
		
		for i in xrange(9):
			a,b = bounds[i]
			if (a<=deg) and (deg<=b): return results[i]
		
		raise "Kanya"
		
	
	def addPoint(self, point):
		
		if self.lastPoint is None:
			self.lastPoint = point
			self.previousPoint = point
			#if self.dbg: print "lastPoint updated"
			return
		
		dx = point[0] - self.lastPoint[0]
		dy = point[1] - self.lastPoint[1]
		
		if self.previousPoint == point:
			
			self.zcounter += 1
			
			if self.zcounter > self.zcutoff and not self.inZero:
				self.inZero = True
				if self.dbg: print "Z:", self.zcounter
				# add new vector
				deltaLen = self.myLen((dx, dy))
				if deltaLen > self.cutoff:
					thisVector = self.getVector((dx,dy));
					if self.dbg: print "VEC", thisVector
					self.vectors.append( [thisVector[:], deltaLen, point] )
				
				self.lastPoint = point
		else:
			#print "nonzero-dx", self.k, dx, dy
			self.inZero = False			
			self.zcounter = 0
		
		self.previousPoint = point
			
	
	def returnVector(self):
		ret = []
		# collapse same vectors
		for vec in self.vectors:
			if len(ret) and (ret[-1][0] == vec[0]):
				ret[-1][1] += vec[1]	# add length
				ret[-1][2] = vec[2]	# update point
			else:
				ret.append( vec )
					
		return ret

